@@ -215,10 +215,10 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         gnomeglade.Component.__init__(self, paths.share_dir("glade2/dirdiff.glade"), "dirdiff")
         self.toolbar.set_style( self.prefs.get_toolbar_style() )
         self._map_widgets_into_lists( ["treeview", "fileentry", "diffmap", "scrolledwindow", "linkmap"] )
-        self.lock = 0
         self.popup_menu = DirDiffMenu(self)
         self.set_num_panes(num_panes)
         self.on_treeview_focus_out_event(None, None)
+        self.treeview_focussed = None
 
         for i in range(3):
             self.treeview[i].get_selection().set_mode(gtk.SELECTION_MULTIPLE)
@@ -247,7 +247,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         for r in [ misc.ListItem(i) for i in self.prefs.regexes.split("\n") ]:
             if r.active:
                 try:
-                    self.regexes.append( re.compile(r.value) )
+                    self.regexes.append( re.compile(r.value+"(?m)") )
                 except re.error, e:
                     misc.run_dialog(
                         text=_("Error converting pattern '%s' to regular expression") % r.value )
@@ -285,14 +285,14 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             self.update_regexes()
 
     def _do_to_others(self, master, objects, methodname, args):
-        if self.lock == 0:
-            self.lock = 1
+        if not hasattr(self, "do_to_others_lock"):
+            self.do_to_others_lock = 1
             try:
                 for o in filter(lambda x:x!=master, objects[:self.num_panes]):
                     method = getattr(o,methodname)
                     method(*args)
             finally:
-                self.lock = 0
+                delattr(self, "do_to_others_lock")
 
     def _sync_vscroll(self, adjustment):
         adjs = map(lambda x: x.get_vadjustment(), self.scrolledwindow)
@@ -318,12 +318,14 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             self._update_item_state(iter)
         else: # nope its gone
             self.model.remove(iter)
+        self._update_diffmaps()
 
     def file_created(self, path, pane):
         iter = self.model.get_iter(path)
         while iter and self.model.get_path(iter) != (0,):
             self._update_item_state( iter )
             iter = self.model.iter_parent(iter)
+        self._update_diffmaps()
 
     def on_fileentry_activate(self, entry):
         locs = [ e.get_full_path(0) for e in self.fileentry[:self.num_panes] ]
@@ -401,7 +403,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                                 self.items[ ci ][pane] = i
                     def get(self):
                         if len(self.bad):
-                            misc.run_dialog(_("You are running a case insensitve comparison on"
+                            misc.run_dialog(_("You are running a case insensitive comparison on"
                                 " a case sensitive filesystem. Some files are not visible:\n%s")
                                 % "\n".join( self.bad ), self.parent )
                         keys = self.items.keys()
@@ -475,20 +477,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
            If it is a folder we recursively open diffs for each non equal file.
         """
         paths = filter(os.path.exists, self.model.value_paths(iter))
-        arefiles = map(os.path.isfile, paths)
-        if 0 not in arefiles:
-            if force or int(self.model.get_state(iter,pane)) >= tree.STATE_NEW:
-                self.emit("create-diff", paths)
-        else:
-            aredirs = map(os.path.isdir, paths)
-            if aredirs:
-                child = self.model.iter_children(iter)
-                while child:
-                    state = int(self.model.get_state(child, pane))
-                    self.launch_comparison(child, pane, force=0)
-                    child = self.model.iter_next(child)
-            else:
-                print "Mixture of files and folders?", paths
+        self.emit("create-diff", paths)
 
     def launch_comparisons_on_selected(self):
         """Launch comparisons on all selected elements.
@@ -531,7 +520,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                                 continue
                         misc.copytree(src, dst)
                         self.recursively_update( path )
-                except OSError, e:
+                except (OSError,IOError), e:
                     misc.run_dialog(_("Error copying '%s' to '%s'\n\n%s.") % (src, dst,e), self)
 
     def delete_selected(self):
@@ -561,6 +550,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
 
     def on_treeview_cursor_changed(self, *args):
         pane = self._get_focused_pane()
+        if pane == None: return
         paths = self._get_selected_paths(pane)
         if len(paths) > 0:
             def rwx(mode):
@@ -581,6 +571,11 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             else:
                 self.emit("status-changed", "%s : %s" % (rwx(stat.st_mode), nice(time.time() - stat.st_mtime) ) )
 
+    def on_switch_event(self):
+        if self.treeview_focussed:
+            self.scheduler.add_task( self.treeview_focussed.grab_focus )
+            self.scheduler.add_task( self.on_treeview_cursor_changed )
+
     def on_treeview_key_press_event(self, view, event):
         pane = self.treeview.index(view)
         tree = None
@@ -627,13 +622,14 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
 
     def on_treeview_row_expanded(self, view, iter, path):
         self._do_to_others(view, self.treeview, "expand_row", (path,0) )
-        self._update_difmaps()
+        self._update_diffmaps()
 
     def on_treeview_row_collapsed(self, view, me, path):
         self._do_to_others(view, self.treeview, "collapse_row", (path,) )
-        self._update_difmaps()
+        self._update_diffmaps()
 
     def on_treeview_focus_in_event(self, tree, event):
+        self.treeview_focussed = tree
         pane = self.treeview.index(tree)
         if pane > 0:
             self.button_copy_left.set_sensitive(1)
@@ -784,11 +780,8 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                 self.model.set_state(iter, j,  tree.STATE_MISSING)
         return different
 
-    def update_diff_maps(self):
-        return
-
     def on_treeview_button_press_event(self, treeview, event):
-        # unselect others
+        # unselect other panes
         for t in filter(lambda x:x!=treeview, self.treeview[:self.num_panes]):
             t.get_selection().unselect_all()
         if event.button == 3:
@@ -798,9 +791,11 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
                 pass # clicked outside tree
             else:
                 treeview.grab_focus()
-                treeview.set_cursor( path, col, 0)
+                selected = self._get_selected_paths( self.treeview.index(treeview) )
+                if len(selected) <= 1 and event.state == 0:
+                    treeview.set_cursor( path, col, 0)
                 self.popup_menu.popup_in_pane( self.treeview.index(treeview) )
-            return 1
+            return event.state==0
 
     def set_num_panes(self, n):
         if n != self.num_panes and n in (1,2,3):
@@ -832,7 +827,7 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
         self.label_text = " : ".join(shortnames)
         self.label_changed()
 
-    def _update_difmaps(self):
+    def _update_diffmaps(self):
         self.diffmap[0].queue_draw()
         self.diffmap[1].queue_draw()
 
@@ -931,23 +926,63 @@ class DirDiff(melddoc.MeldDoc, gnomeglade.Component):
             adj.set_value( max( min(upper, val), 0) )
             return 1
 
-    def on_file_changed(self, filename):
+    def on_file_changed(self, changed_filename):
+        """When a file has changed, try to find it in our tree
+           and update its status if necessary
+        """
         model = self.model
-        iter = model.get_iter_root()
-        for pane,path in misc.enumerate( model.value_paths(iter) ):
-            if filename.startswith(path): 
-                while iter:
-                    child = model.iter_children( iter )
-                    while child:
-                        path = model.value_path(child, pane)
-                        if filename == path:
-                            self._update_item_state(child)
-                            return
-                        elif filename.startswith(path):
-                            break
-                        else:
-                            child = self.model.iter_next( child )
-                    iter = child
+        changed_paths = []
+        # search each panes tree for changed_filename
+        for pane in range(self.num_panes):
+            it = model.get_iter_root()
+            current = model.value_path(it, pane).split(os.sep)
+            changed = changed_filename.split(os.sep)
+            # early exit. does filename begin with root?
+            try:
+                if changed[:len(current)] != current:
+                    continue
+            except IndexError:
+                continue
+            changed = changed[len(current):]
+            # search the tree component at a time
+            for component in changed:
+                child = model.iter_children( it )
+                while child:
+                    leading, name = os.path.split( model.value_path(child, pane) )
+                    if component == name : # found it
+                        it = child
+                        break
+                    else:
+                        child = self.model.iter_next( child ) # next
+                if not it:
+                    break
+            # save if found and unique
+            if it:
+                path = model.get_path(it)
+                if path not in changed_paths:
+                    changed_paths.append(path)
+        # do the update
+        for path in changed_paths:
+            self._update_item_state( model.get_iter(path) )
+
+    def next_diff(self, direction):
+        if self.treeview_focussed:
+            pane = self.treeview.index( self.treeview_focussed )
+        else:
+            pane = 0
+        start_iter = self.model.get_iter( (self._get_selected_paths(pane) or [(0,)])[-1] )
+
+        def goto_iter(it):
+            curpath = self.model.get_path(it)
+            for i in range(len(curpath)-1):
+                self.treeview[pane].expand_row( curpath[:i+1], 0)
+            self.treeview[pane].set_cursor(curpath)
+
+        search = {gdk.SCROLL_UP : self.model.inorder_search_up}.get(direction, self.model.inorder_search_down)
+        for it in search( start_iter ):
+            state = int(self.model.get_state( it, pane ))
+            if state not in (tree.STATE_NORMAL, tree.STATE_EMPTY):
+                goto_iter(it)
                 return
 
 gobject.type_register(DirDiff)
