@@ -43,7 +43,7 @@ try:
 except ImportError:
     pass
 
-version = "0.9.3"
+version = "0.9.4"
 
 # magic developer switch, changes some behaviour
 developer = 0
@@ -254,6 +254,7 @@ class PreferencesDialog(gnomeglade.Component):
         cols = [ ("Name", type("")), ("Active", type(0)), ("Regex", type("")) ]
         self.textfilter = ListWidget( cols, self.prefs, "regexes")
         self.text_filters_box.pack_start(self.textfilter.widget)
+        self.checkbutton_ignore_blank_lines = self.prefs.ignore_blank_lines
         # encoding
         self.entry_text_codecs.set_text( self.prefs.text_codecs )
         self._map_widgets_into_lists( ["save_encoding"] )
@@ -315,6 +316,8 @@ class PreferencesDialog(gnomeglade.Component):
     #
     # filters
     #
+    def on_checkbutton_ignore_blank_lines_toggled(self, check):
+        self.prefs.ignore_blank_lines = check.get_active()
 
     #
     # encoding
@@ -459,9 +462,12 @@ class MeldPreferences(prefs.Preferences):
                                               "Binaries\t1\t*.{pyc,a,obj,o,so,la,lib,dll}\n" + \
                                               "Media\t0\t*.{jpg,gif,png,wav,mp3,ogg,xcf,xpm}"),
         "regexes" : prefs.Value(prefs.STRING, "CVS keywords\t0\t\$[^:]+:[^\$]+\$\n" + \
-                                              "C comment\t0\t/\*[^*]*\*+([^/*][^*]*\*+)*/\n" + \
                                               "C++ comment\t0\t//.*\n" + \
-                                              "Script comment\t0\t#.*")
+                                              "C comment\t0\t/\*[^*]*\*+([^/*][^*]*\*+)*/\n" + \
+                                              "All whitespace\t0\t[ \\t\\r\\f\\v]*\n" + \
+                                              "Leading whitespace\t0\t^[ \\t\\r\\f\\v]*\n" + \
+                                              "Script comment\t0\t#.*"),
+        "ignore_blank_lines" : prefs.Value(prefs.BOOL, 1)
     }
 
     def __init__(self):
@@ -552,7 +558,7 @@ class MeldApp(gnomeglade.GnomeApp):
     def on_idle(self):
         ret = self.scheduler.iteration()
         if ret:
-            if isinstance(ret, basestring):
+            if type(ret) in (type(""), type(u"")):
                 self.statusbar.set_task_status(ret)
             elif type(ret) == type(0.0):
                 self.statusbar.task_progress.set_fraction(ret)
@@ -611,6 +617,8 @@ class MeldApp(gnomeglade.GnomeApp):
     #
     # Toolbar and menu items (file)
     #
+    def on_menu_file_new_activate(self, *extra):
+        NewDocDialog(self, NewDocDialog.TYPE.DIFF2)
     def on_menu_new_diff2_activate(self, *extra):
         NewDocDialog(self, NewDocDialog.TYPE.DIFF2)
     def on_menu_new_diff3_activate(self, *extra):
@@ -774,17 +782,22 @@ class MeldApp(gnomeglade.GnomeApp):
         doc.set_files(files)
 
     def append_diff(self, paths):
-        if 0 not in [os.path.isfile(p) for p in paths]:
-            self.append_filediff(paths)
-        elif 0 not in [os.path.isdir(p) for p in paths]:
-            self.append_dirdiff(paths)
-        else:
+        aredirs = [ os.path.isdir(p) for p in paths ]
+        arefiles = [ os.path.isfile(p) for p in paths ]
+        if (1 in aredirs) and (1 in arefiles):
             m = _("Cannot compare a mixture of files and directories.\n")
-            for p in paths:
-                m += "\n".join( ["(%s)\t`%s'" % (os.path.isfile(p) and _("file") or _("folder"), p) for p in paths] )
+            for i in range(len(paths)):
+                status = aredirs[i] and _("folder") \
+                    or arefiles[i] and _("file") \
+                    or _("nonexistant")
+                m += "\n(%s)\t`%s'" % (status, paths[i])
             misc.run_dialog( m,
                     parent = self,
                     buttonstype = gtk.BUTTONS_OK)
+        elif 1 in aredirs:
+            self.append_dirdiff(paths)
+        else:
+            self.append_filediff(paths)
 
     def append_cvsview(self, locations):
         assert len(locations) in (1,)
@@ -872,16 +885,14 @@ def main():
             return
 
     app = MeldApp()
-    arg = sys.argv[1:]
+    args = sys.argv[1:]
 
-    if len(arg) == 0:
+    if len(args) == 0:
         pass
 
-    elif len(arg) == 1:
-        a = arg[0]
-        if os.path.isdir(a):
-            app.append_cvsview( [a] )
-        elif os.path.isfile(a):
+    elif len(args) == 1:
+        a = args[0]
+        if os.path.isfile(a):
             doc = cvsview.CvsView(app.prefs)
             def cleanup():
                 app.scheduler.remove_scheduler(doc.scheduler)
@@ -891,35 +902,15 @@ def main():
             doc.connect("create-diff", lambda obj,arg: app.append_diff(arg) )
             doc.run_cvs_diff([a])
         else:
-            app.usage( _("`%s' is not a directory or file, cannot open cvs view") % arg[0])
+            if os.path.exists( os.path.join(a,".svn") ):
+                app.append_svnview( [a] )
+            else:
+                app.append_cvsview( [a] )
                 
-    elif len(arg) in (2,3):
-        done = 0
-        exists = map( lambda a: os.access(a, os.R_OK), arg)
-        if 0 in exists:
-            m = _("Cannot open ")
-            for i in range(len(arg)):
-                if not exists[i]:
-                    m += "`%s'" % arg[i]
-            app.usage(m)
-            done = 1
-        if not done:
-            arefiles = map( os.path.isfile, arg)
-            if 0 not in arefiles:
-                app.append_diff( arg )
-                done = 1
-        if not done:
-            aredirs = map( os.path.isdir, arg)
-            if 0 not in aredirs:
-                app.append_dirdiff( arg )
-                done = 1
-        if not done:
-            m = _("Cannot compare a mixture of files and directories.\n")
-            for i in range(len(arg)):
-                m += "(%s)\t`%s'\n" % (arefiles[i] and _("file") or _("folder"), arg[i])
-            app.usage(m)
+    elif len(args) in (2,3):
+        app.append_diff(args)
     else:
         app.usage( _("Wrong number of arguments (Got %i)") % len(arg))
 
-    app.mainloop()
+    app.main()
 
