@@ -32,9 +32,9 @@ STATE_MODIFIED, STATE_CONFLICT, STATE_REMOVED, \
 STATE_MISSING, STATE_MAX = range(12)
 
 load = lambda x,s=14: gnomeglade.load_pixbuf( paths.share_dir("glade2/pixmaps",x), s)
-pixbuf_folder = load("tree-folder-normal.png")
-pixbuf_folder_new = load("tree-folder-new.png")
-pixbuf_folder_changed = load("tree-folder-changed.png")
+pixbuf_folder = load("tree-folder-normal.png", 20)
+pixbuf_folder_new = load("tree-folder-new.png", 20)
+pixbuf_folder_changed = load("tree-folder-changed.png", 20)
 pixbuf_file = load("tree-file-normal.png")
 pixbuf_file_new = load("tree-file-new.png")
 pixbuf_file_changed = load("tree-file-changed.png")
@@ -119,3 +119,44 @@ class DiffTreeStore(gtk.TreeStore):
         STATE = self.column_index(COL_STATE, pane)
         return self.get_value(iter, STATE)
 
+    def inorder_search_down(self, it):
+        while it:
+            child = self.iter_children(it)
+            if child:
+                it = child
+            else:
+                next = self.iter_next(it)
+                if next:
+                    it = next
+                else:
+                    while 1:
+                        it = self.iter_parent(it)
+                        if it:
+                            next = self.iter_next(it)
+                            if next:
+                                it = next
+                                break
+                        else:
+                            raise StopIteration()
+            yield it
+
+    def inorder_search_up(self, it):
+        while it:
+            path = self.get_path(it)
+            if path[-1]:
+                path = path[:-1] + (path[-1]-1,)
+                it = self.get_iter(path)
+                while 1:
+                    nc = self.iter_n_children(it)
+                    if nc:
+                        it = self.iter_nth_child(it, nc-1)
+                    else:
+                        break
+            else:
+                up = self.iter_parent(it)
+                if up:
+                    it = up
+                else:
+                    raise StopIteration()
+            yield it
+
